/* Copyright (c) 2001, Nokia Mobile Phones. All rights reserved */

/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#ifndef __SOCKETSENGINE_H__
#define __SOCKETSENGINE_H__

#include <in_sock.h>
#include <eikapp.h>
#include <eikdoc.h>
#include <e32std.h>
#include <aknwaitdialog.h>  
#include "TimeOutNotify.h"
#include "Sockets.hrh"
#include "uinotify.h"

class CHttpRead;
class CHttpWrite;
class CTimeOutTimer;


/*! 
  @class CSocketsEngine
  
  @discussion This class is the main engine part of the sockets application.
  It establishes a TCP connection using its server name and port number (performing a DNS lookup
  operation first, if appropriate).
  It creates instances of separate active objects to perform reading from, and writing to, the socket.
  */
class CHttpEngine : public CActive, public MTimeOutNotify, public MProgressDialogCallback
    {
public: // new methods
/*!
  @function NewL
   
  @discussion Create a CSocketsEngine object
  @param aConsole console to use for ui output
  @result a pointer to the created instance of CSocketsEngine
  */
    static CHttpEngine* NewL(MHttpNotify& aConsole);

/*!
  @function NewLC
   
  @discussion Create a CSocketsEngine object
  @param aConsole console to use for ui output
  @result a pointer to the created instance of CSocketsEngine
  */
    static CHttpEngine* NewLC(MHttpNotify& aConsole);

/*!
  @function ~CSocketsEngine
  
  @discussion Destroy the object and release all memory objects
  */
    ~CHttpEngine();


    void StartHttpGetL(const TDesC& aServerName, const TDesC8& aPath, TInt aPort);

    void DialogDismissedL(TInt aButtonId);
    
    void Disconnected();

/*!
  @function Disconnect

  @discussion Disconnect socket
  */
    void Disconnect();

/*!
  @function WriteL

  @discussion Write data to socket
  @param aData data to be written
  */
    void WriteL(const TDesC8& aData);

/*!
  @function Read

  @discussion Initiate read of data from socket
  */
    void Read();


/*!
  @function Connected

  @discussion Is socket fully connected?
  @result true if socket is connected
  */
    TBool Connected() const;

public: // from MTimeOutNotify
/*!
  @function TimerExpired

  @discussion The function to be called when a timeout occurs
  */
    void TimerExpired(); 

protected: // from CActive
/*!
  @function DoCancel
   
  @discussion cancel any outstanding operation
  */
    void DoCancel();

/*!
  @function RunL
   
  @discussion called when operation complete
  */
    void RunL();

private: // New methods
/*!
  @function CSocketsEngine
  
  @discussion Perform the first phase of two phase construction 
  */
    CHttpEngine(MHttpNotify& aConsole);

/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CSocketsEngine 
  @param aConsole the console to use for ui output
  */
    void ConstructL();

/*!
  @function ConnectL
   
  @discussion initiate a connect operation on a socket
  @param aAddr the ip address to connect to
  */
    void ConnectL(TUint32 aAddr);

    enum TSocketsEngineState 
        {
        ENotConnected,
        EConnecting,
        EConnected,
        EWriting,
        EReading,
        ETimedOut, 
        ELookingUp,
        ELookUpFailed,
        EConnectFailed,
        EDisconnecting
        };

/*!
  @function ChangeStatus
   
  @discussion handle a change in this object's status
  @param aNewStatus new status
  */
    void ChangeStatus(TSocketsEngineState aNewStatus);


private:
    // Member variables
    TSocketsEngineState         iEngineStatus; // this object's current status
    MHttpNotify&                  iNotify;
    CHttpRead*               iSocketsRead; // socket reader active object
    CHttpWrite*              iSocketsWrite; // socket writer active object
    RSocket                     iSocket; // the actual socket
    RSocketServ                 iSocketServ; // the socket server
    RHostResolver               iResolver; // DNS name resolver
    TNameEntry                  iNameEntry;
    TNameRecord                 iNameRecord;
    CTimeOutTimer*              iTimer; // timer active object
    TInetAddr                   iAddress;

    TBuf8<128> iUrl;
    TInt iPort;
    TBuf<KMaxServerNameLength>  iServerName; // server name to connect to
    TBuf8<KMaxRequestLength> iHttpRequest;
    
    };

#endif // __SOCKETSENGINE_H__

