/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

/* Based on Nokia sample code */
/* Copyright (c) 2001, Nokia Mobile Phones. All rights reserved */



#include "httpread.h"
#include "UINotify.h"
#include "utf.h"

#include <weather.rsg>


CHttpRead* CHttpRead::NewL(MHttpNotify& aConsole, CHttpEngine& aEngine, RSocket& aSocket)
    {

    CHttpRead* self = new (ELeave) CHttpRead(aConsole, aEngine, aSocket);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    return self;
    }

    


CHttpRead::CHttpRead(MHttpNotify& aNotify, CHttpEngine& aEngine, RSocket& aSocket)
: CActive(EPriorityStandard),
  iSocket(aSocket),
  iNotify(aNotify),
  iEngine(aEngine)
    {
    }


CHttpRead::~CHttpRead()
    {
    Cancel();
    }


void CHttpRead::ConstructL()
    {
    CActiveScheduler::Add(this);
    }

void CHttpRead::DoCancel()
    {
    // Cancel asychronous read request
    iSocket.CancelRead();

    iSocket.CancelAll();

    RDebug::Print(_L("cancelled\n"));
    }



_LIT(KEol, "\r\n");  
TInt CHttpRead::ExtractLine(TPtrC aBuffer, TPtrC& aLine)
    {
    TInt offset = aBuffer.Find(KEol);
    if (offset == KErrNotFound) return KErrNotFound;

    aLine.Set(aBuffer.Ptr(), offset);
    RDebug::Print(_L("got <%S>\n"), &aLine);

    return offset+2;
    }


_LIT(KResponseCode, "HTTP*\r\n*");
_LIT(KBodyStart, "\r\n\r\n");
_LIT(KContentType, "Content-Type: "); 
void CHttpRead::ProcessResponse()
    {
    
    TInt offset = iReadBuffer16.Match(KResponseCode());
    
    if (offset == KErrNotFound)
        {

        iNotify.SocketStatusNotify(R_TEXT_NOT_A_HTTP_RESPONSE, EFalse);
        return;
        }
    else
        {
        TPtrC respCodeStr = iReadBuffer16.Mid(9, 3);
        TLex lexer(respCodeStr);
        if (lexer.Val(iResponseCode) != KErrNone)
            {
            //this probably won't happen
            iNotify.SocketStatusNotify(_L("Problem with response code"), EFalse);
            return;
            }
        RDebug::Print(_L("got %S %d offset=%d\n"), &respCodeStr, iResponseCode, offset);
        
        }
    
    TInt bodyStartIndex = iReadBuffer16.Find(KBodyStart);
    if (bodyStartIndex == KErrNotFound)
        {
        RDebug::Print(_L("erm..."));
        }

    iBody.Set(iReadBuffer16.Ptr()+bodyStartIndex, iReadBuffer16.Length()-bodyStartIndex);

    RDebug::Print(_L("body start=%d:\n------------------\n%S\n------------------\n"), bodyStartIndex, &iBody);

    TInt eolIndex = 0;
    TPtrC line;
    while ((offset < bodyStartIndex) && (eolIndex != KErrNotFound))
        {
        eolIndex = ExtractLine(iReadBuffer16.Mid(offset), line);

        if (line.Find(KContentType) != KErrNotFound)
            {
            iContentType.Set(line.Ptr()+KContentType().Length(), line.Length()-KContentType().Length());
            RDebug::Print(_L("content type :[%S]\n"), &iContentType);
            }

        offset += eolIndex;
        }

    }


void CHttpRead::RunL()
    {
    // Active object request complete handler
    if (iStatus == KErrNone)
        {

        //iNotify.SocketStatusNotify(_L("Reading data"), EFalse);
        if ((iReadBuffer16.Length() + iTempBuffer8.Length()) >= (iReadBuffer16.MaxLength()-KTempBufferSize))
            {
            RDebug::Print(_L("full!\n"));
            DoCancel();
            ProcessResponse();
            iNotify.HttpLoadedNotify(iBody, iResponseCode);
            iEngine.Disconnected();
            return;
            }

        CnvUtfConverter::ConvertToUnicodeFromUtf8(iTempBuffer16, iTempBuffer8);

        iReadBuffer16.Append(iTempBuffer16);
        RDebug::Print(_L("adding %d chars, length is now %d [%S]\n"), iTempBuffer16.Length(), iReadBuffer16.Length(), &iTempBuffer16);



        IssueRead(); // Immediately start another read
        }
    else if ((iStatus == KErrEof) || (iStatus == KErrDisconnected))
        {
        iNotify.SocketStatusNotify(R_TEXT_FINISHED_READING, EFalse);
        DoCancel();
        ProcessResponse();
        iNotify.HttpLoadedNotify(iBody, iResponseCode);
        iEngine.Disconnected();
        }
    else
        {
        DoCancel();
        TBuf<60> errorBuf;
        errorBuf.Format(_L("error reading from socket. status=%d"), iStatus);
        iNotify.SocketStatusNotify(errorBuf, ETrue);

        iEngine.Disconnected();

        }    
    }

void CHttpRead::IssueRead()
    {
    // Initiate a new read from socket into iBuffer
    ASSERT(!IsActive());
    iSocket.RecvOneOrMore(iTempBuffer8, 0, iStatus, iDummyLength);

    SetActive();
    }

void CHttpRead::Start()
    {

    iReadBuffer16.Zero();
    // Initiate a new read from socket into iBuffer
    if (!IsActive())
        {
        IssueRead();
        }
    }
